

#include "base/SMPDataFlowAnalysis.h"
#include <interfaces/abstract/STARSOp.h>
#include <interfaces/SMPDBInterface.h>
#include <interfaces/STARSTypes.h>


void STARS_op_t::MDExtractAddressFields(int &BaseReg, int &IndexReg, uint16_t &Scale, STARS_ea_t &Offset) {
	assert(this->IsMemOp());

	Scale = 0;
	BaseReg = STARS_x86_R_none;
	IndexReg = STARS_x86_R_none;
	Offset = this->GetAddr();

	if (this->HasSIBByte()) {
		BaseReg = this->MDGetSIBBaseReg();
		IndexReg = (int) this->MDGetSIBIndexReg();
		if (MD_STACK_POINTER_REG == IndexReg) // signifies no index register
			IndexReg = STARS_x86_R_none;
		if (STARS_x86_R_none != IndexReg) {
			Scale = (uint16_t) this->GetSIBScaleFactor();
		}
		if (STARS_x86_R_none != BaseReg) {
			if (this->IsStaticMemOp()) {
				BaseReg = STARS_x86_R_none;
				// Only IndexReg is allowed for o_mem with SIB byte
			}
		}
	}
	else { // no SIB byte; can have base reg but no index reg or scale factor
		BaseReg = (int) this->GetReg();  // cannot be STARS_x86_R_none for no SIB case
		if (this->IsStaticMemOp()) {
			BaseReg = STARS_x86_R_none; // no Base register for o_mem operands
		}
	}
	return;
} // end of STARS_op_t::MDExtractAddressFields()

// Get rid of unnecessary SIB byte encodings, etc.
void STARS_op_t::CleanOpndEncoding(void) {
	if (this->IsMemOp()) {
		int BaseReg;
		int IndexReg;
		uint16_t ScaleFactor;
		STARS_ea_t offset;
		this->MDExtractAddressFields(BaseReg, IndexReg, ScaleFactor, offset);
		if (this->HasSIBByte()) {
			if ((IndexReg == STARS_x86_R_none) && (BaseReg != STARS_x86_R_none)) {
				// No need for a SIB byte if just base reg or base reg + offset.
				this->ClearSIB();
				this->SetReg((STARS_regnum_t) BaseReg);
			}
		}
		if ((0 == offset) && this->IsMemDisplacementOp()) {
			this->SetTypeToMemNoDisplacement();
		}
		else if ((0 != offset) && this->IsMemNoDisplacementOp()) {
			this->SetTypeToMemDisplacement();
		}
	}
	return;
}

